import { type HD } from "@bsv/sdk";
import { type APIFetcher } from "./api";
import type { Identity, IdentityAttribute, IdentityAttributes } from "./interface";
import type { GetAttestationResponse } from "./apiTypes";
/**
 * BAP_ID class
 *
 * This class should be used in conjunction with the BAP class
 *
 * @type {BAP_ID}
 */
declare class BAP_ID {
    #private;
    idName: string;
    description: string;
    rootAddress: string;
    identityKey: string;
    identityAttributes: IdentityAttributes;
    getApiData: APIFetcher;
    constructor(HDPrivateKey: HD, identityAttributes?: IdentityAttributes, idSeed?: string);
    set BAP_SERVER(bapServer: string);
    get BAP_SERVER(): string;
    set BAP_TOKEN(token: string);
    get BAP_TOKEN(): string;
    deriveIdentityKey(address: string): string;
    /**
     * Helper function to parse identity attributes
     *
     * @param identityAttributes
     * @returns {{}}
     */
    parseAttributes(identityAttributes: IdentityAttributes | string): IdentityAttributes;
    /**
     * Parse a text of urn string into identity attributes
     *
     * urn:bap:id:name:John Doe:e2c6fb4063cc04af58935737eaffc938011dff546d47b7fbb18ed346f8c4d4fa
     * urn:bap:id:birthday:1990-05-22:e61f23cbbb2284842d77965e2b0e32f0ca890b1894ca4ce652831347ee3596d9
     * urn:bap:id:over18:1:480ca17ccaacd671b28dc811332525f2f2cd594d8e8e7825de515ce5d52d30e8
     *
     * @param urnIdentityAttributes
     */
    parseStringUrns(urnIdentityAttributes: string): IdentityAttributes;
    /**
     * Returns the identity key
     *
     * @returns {*|string}
     */
    getIdentityKey(): string;
    /**
     * Returns all the attributes in the identity
     *
     * @returns {*}
     */
    getAttributes(): IdentityAttributes;
    /**
     * Get the value of the given attribute
     *
     * @param attributeName
     * @returns {{}|null}
     */
    getAttribute(attributeName: string): IdentityAttribute | null;
    /**
     * Set the value of the given attribute
     *
     * If an empty value ('' || null || false) is given, the attribute is removed from the ID
     *
     * @param attributeName string
     * @param attributeValue any
     * @returns {{}|null}
     */
    setAttribute(attributeName: string, attributeValue: string | Record<string, string>): void;
    /**
     * Unset the given attribute from the ID
     *
     * @param attributeName
     * @returns {{}|null}
     */
    unsetAttribute(attributeName: string): void;
    /**
     * Get all attribute urn's for this id
     *
     * @returns {string}
     */
    getAttributeUrns(): string;
    /**
     * Create an return the attribute urn for the given attribute
     *
     * @param attributeName
     * @returns {string|null}
     */
    getAttributeUrn(attributeName: string): string | null;
    /**
     * Add an attribute to this identity
     *
     * @param attributeName
     * @param value
     * @param nonce
     */
    addAttribute(attributeName: string, value: any, nonce?: string): void;
    /**
     * This should be called with the last part of the signing path (/.../.../...)
     * This library assumes the first part is m/424150'/0'/0' as defined at the top of this file
     *
     * @param path The second path of the signing path in the format [0-9]{0,9}/[0-9]{0,9}/[0-9]{0,9}
     */
    set rootPath(path: string);
    get rootPath(): string;
    getRootPath(): string;
    /**
     * This should be called with the last part of the signing path (/.../.../...)
     * This library assumes the first part is m/424150'/0'/0' as defined at the top of this file
     *
     * @param path The second path of the signing path in the format [0-9]{0,9}/[0-9]{0,9}/[0-9]{0,9}
     */
    set currentPath(path: string);
    get currentPath(): string;
    get previousPath(): string;
    /**
     * This can be used to break the deterministic way child keys are created to make it harder for
     * an attacker to steal the identites when the root key is compromised. This does however require
     * the seeds to be stored at all times. If the seed is lost, the identity will not be recoverable.
     */
    get idSeed(): string;
    /**
     * Increment current path to a new path
     *
     * @returns {*}
     */
    incrementPath(): void;
    /**
     * Check whether the given path is a valid path for use with this class
     * The signing paths used here always have a length of 3
     *
     * @param path The last part of the signing path (example "/0/0/1")
     * @returns {boolean}
     */
    validatePath(path: string): boolean;
    /**
     * Get the OP_RETURN for the initial ID transaction (signed with root address)
     *
     * @returns {[]}
     */
    getInitialIdTransaction(): string[];
    /**
     * Get the OP_RETURN for the ID transaction of the current address / path
     *
     * @returns {[]}
     */
    getIdTransaction(previousPath?: string): string[];
    /**
     * Get address for given path
     *
     * @param path
     * @returns {*}
     */
    getAddress(path: string): string;
    /**
     * Get current signing address
     *
     * @returns {*}
     */
    getCurrentAddress(): string;
    /**
     * Get the public key for encrypting data for this identity
     */
    getEncryptionPublicKey(): string;
    /**
     * Get the public key for encrypting data for this identity, using a seed for the encryption
     */
    getEncryptionPublicKeyWithSeed(seed: string): string;
    /**
     * Encrypt the given string data with the identity encryption key
     * @param stringData
     * @param counterPartyPublicKey Optional public key of the counterparty
     * @return string Base64
     */
    encrypt(stringData: string, counterPartyPublicKey?: string): string;
    /**
     * Decrypt the given ciphertext with the identity encryption key
     * @param ciphertext
     */
    decrypt(ciphertext: string, counterPartyPublicKey?: string): string;
    /**
     * Encrypt the given string data with the identity encryption key
     * @param stringData
     * @param seed String seed
     * @param counterPartyPublicKey Optional public key of the counterparty
     * @return string Base64
     */
    encryptWithSeed(stringData: string, seed: string, counterPartyPublicKey?: string): string;
    /**
     * Decrypt the given ciphertext with the identity encryption key
     * @param ciphertext
     * @param seed String seed
    //  * @param counterPartyPublicKey Public key of the counterparty
     */
    decryptWithSeed(ciphertext: string, seed: string, counterPartyPublicKey?: string): string;
    private getEncryptionPrivateKeyWithSeed;
    /**
     * Get an attestation string for the given urn for this identity
     *
     * @param urn
     * @returns {string}
     */
    getAttestation(urn: string): string;
    /**
     * Generate and return the attestation hash for the given attribute of this identity
     *
     * @param attribute Attribute name (name, email etc.)
     * @returns {string}
     */
    getAttestationHash(attribute: string): string | null;
    /**
     * Sign a message with the current signing address of this identity
     *
     * @param message
     * @param signingPath
     * @returns {{address: string, signature: string}}
     */
    signMessage(message: string | Buffer, signingPath?: string): {
        address: string;
        signature: string;
    };
    /**
     * Sign a message using a key based on the given string seed
     *
     * This works by creating a private key from the root key of this identity. It will always
     * work with the rootPath / rootKey, to be deterministic. It will not change even if the keys
     * are rotated for this ID.
     *
     * This is used in for instance deterministic login systems, that do not support BAP.
     *
     * @param message
     * @param seed {string} String seed that will be used to generate a path
     */
    signMessageWithSeed(message: string, seed: string): {
        address: string;
        signature: string;
    };
    /**
     * Sign an op_return hex array with AIP
     * @param opReturn {array}
     * @param signingPath {string}
     * @param outputType {string}
     * @return {[]}
     */
    signOpReturnWithAIP(opReturn: string[], signingPath?: string, outputType?: BufferEncoding): string[];
    /**
     * Construct an AIP buffer from the op return data
     * @param opReturn
     * @returns {Buffer}
     */
    getAIPMessageBuffer(opReturn: string[]): Buffer;
    /**
     * Get all signing keys for this identity
     */
    getIdSigningKeys(): Promise<any>;
    /**
     * Get all attestations for the given attribute
     *
     * @param attribute
     */
    getAttributeAttestations(attribute: string): Promise<GetAttestationResponse>;
    /**
     * Import an identity from a JSON object
     *
     * @param identity{{}}
     */
    import(identity: Identity): void;
    /**
     * Export this identity to a JSON object
     * @returns {{}}
     */
    export(): Identity;
}
export { BAP_ID };
