import { type APIFetcher } from "./api";
import type { GetAttestationResponse, GetIdentityByAddressResponse } from "./apiTypes";
import { BAP_ID } from "./id";
import type { Attestation, Identity, IdentityAttributes, PathPrefix } from "./interface";
type Identities = {
    lastIdPath: string;
    ids: Identity[];
};
/**
 * BAP class
 *
 * Creates an instance of the BAP class and uses the given HDPrivateKey for all BAP operations.
 *
 * @param HDPrivateKey
 */
export declare class BAP {
    #private;
    getApiData: APIFetcher;
    constructor(HDPrivateKey: string, token?: string, server?: string);
    get lastIdPath(): string;
    /**
     * Get the public key of the given childPath, or of the current HDPrivateKey of childPath is empty
     *
     * @param childPath Full derivation path for this child
     * @returns {*}
     */
    getPublicKey(childPath?: string): string;
    /**
     * Get the public key of the given childPath, or of the current HDPrivateKey of childPath is empty
     *
     * @param childPath Full derivation path for this child
     * @returns {*}
     */
    getHdPublicKey(childPath?: string): string;
    set BAP_SERVER(bapServer: string);
    get BAP_SERVER(): string;
    set BAP_TOKEN(token: string);
    get BAP_TOKEN(): string;
    /**
     * This function verifies that the given bapId matches the given root address
     * This is used as a data integrity check
     *
     * @param bapId BAP_ID instance
     */
    checkIdBelongs(bapId: BAP_ID): boolean;
    /**
     * Returns a list of all the identity keys that are stored in this instance
     *
     * @returns {string[]}
     */
    listIds(): string[];
    /**
     * Create a new Id and link it to this BAP instance
     *
     * This function uses the length of the #ids of this class to determine the next valid path.
     * If not all ids related to this HDPrivateKey have been loaded, determine the path externally
     * and pass it to newId when creating a new ID.
     *
     * @param path
     * @param identityAttributes
     * @param idSeed
     * @returns {*}
     */
    newId(path?: string, identityAttributes?: IdentityAttributes, idSeed?: string): BAP_ID;
    /**
     * Remove identity
     *
     * @param idKey
     * @returns {*}
     */
    removeId(idKey: string): void;
    /**
     * Get the next valid path for the used HDPrivateKey and loaded #ids
     *
     * @returns {string}
     */
    getNextValidPath(): PathPrefix;
    /**
     * Get a certain Id
     *
     * @param identityKey
     * @returns {null}
     */
    getId(identityKey: string): BAP_ID | null;
    /**
     * This function is used when manipulating ID's, adding or removing attributes etc
     * First create an id through this class and then use getId to get it. Then you can add/edit or
     * increment the signing path and then re-set it with this function.
     *
     * Note: when you getId() from this class, you will be working on the same object as this class
     * has and any changes made will be propagated to the id in this class. When you call exportIds
     * your new changes will also be included, without having to setId().
     *
     * @param bapId
     */
    setId(bapId: BAP_ID): void;
    /**
     * This function is used to import IDs and attributes from some external storage
     *
     * The ID information should NOT be stored together with the HD private key !
     *
     * @param idData Array of ids that have been exported
     * @param encrypted Whether the data should be treated as being encrypted (default true)
     */
    importIds(idData: Identities | string, encrypted?: boolean): void;
    importEncryptedIds(idData: string): void;
    importOldIds(idData: Identity[]): void;
    /**
     * Export all the IDs of this instance for external storage
     *
     * By default this function will encrypt the data, using a derivative child of the main HD key
     *
     * @param encrypted Whether the data should be encrypted (default true)
     * @returns {[]|*}
     */
    exportIds(encrypted?: true): string;
    exportIds(encrypted: false): Identities;
    /**
     * Export a given ID from this instance for external storage
     *
     * By default this function will encrypt the data, using a derivative child of the main HD key
     *
     * @param idKey The key of the identity to export
     * @param encrypted Whether the data should be encrypted (default true)
     * @returns {[]|*}
     */
    exportId(idKey: string, encrypted?: true): string;
    exportId(idKey: string, encrypted: false): Identities;
    /**
     * Encrypt a string of data
     *
     * @param string
     * @returns {string}
     */
    encrypt(string: string): string;
    /**
     * Decrypt a string of data
     *
     * @param string
     * @returns {string}
     */
    decrypt(string: string): string;
    /**
     * Sign an attestation for a user
     *
     * @param attestationHash The computed attestation hash for the user - this should be calculated with the BAP_ID class for an identity for the user
     * @param identityKey The identity key we are using for the signing
     * @param counter
     * @param dataString Optional data string that will be appended to the BAP attestation
     * @returns {string[]}
     */
    signAttestationWithAIP(attestationHash: string, identityKey: string, counter?: number, dataString?: string): string[];
    /**
     * Verify an AIP signed attestation for a user
     *
     * [
     *   '0x6a',
     *   '0x31424150537561506e66476e53424d33474c56397968785564596534764762644d54',
     *   '0x415454455354',
     *   '0x33656166366361396334313936356538353831366439336439643034333136393032376633396661623034386333633031333663343364663635376462383761',
     *   '0x30',
     *   '0x7c',
     *   '0x313550636948473232534e4c514a584d6f5355615756693757537163376843667661',
     *   '0x424954434f494e5f4543445341',
     *   '0x31477531796d52567a595557634638776f6f506a7a4a4c764d383550795a64655876',
     *   '0x20ef60c5555001ddb1039bb0f215e46571fcb39ee46f48b089d1c08b0304dbcb3366d8fdf8bafd82be24b5ac42dcd6a5e96c90705dd42e3ad918b1b47ac3ce6ac2'
     * ]
     *
     * @param tx Array of hex values for the OP_RETURN values
     * @returns {{}}
     */
    verifyAttestationWithAIP(tx: string[]): Attestation;
    /**
     * For BAP attestations we use all fields for the attestation
     *
     * @param attestationHash
     * @param counter
     * @param address
     * @param signature
     * @param dataString Optional data string that will be appended to the BAP attestation
     * @returns {[string]}
     */
    createAttestationTransaction(attestationHash: string, counter: number, address: string, signature: string, dataString?: string): string[];
    /**
     * This is a re-creation of how the bitcoinfiles-sdk creates a hash to sign for AIP
     *
     * @param attestationHash
     * @param counter
     * @param dataString Optional data string
     * @returns {Buffer}
     */
    getAttestationBuffer(attestationHash: string, counter?: number, dataString?: string): Buffer;
    /**
     * Verify that the identity challenge is signed by the address
     *
     * @param message Buffer or utf-8 string
     * @param address Bitcoin address of signee
     * @param signature Signature base64 string
     *
     * @return boolean
     */
    verifySignature(message: string | Buffer, address: string, signature: string): boolean;
    /**
     * Check whether the given transaction (BAP OP_RETURN) is valid, is signed and that the
     * identity signing is also valid at the time of signing
     *
     * @param idKey
     * @param address
     * @param challenge
     * @param signature
     *
     * @returns {Promise<boolean|*>}
     */
    verifyChallengeSignature(idKey: string, address: string, challenge: string, signature: string): Promise<boolean>;
    /**
     * Check whether the given transaction (BAP OP_RETURN) is valid, is signed and that the
     * identity signing is also valid at the time of signing
     *
     * @param tx
     * @returns {Promise<boolean|*>}
     */
    isValidAttestationTransaction(tx: string[]): Promise<any>;
    /**
     * Get all signing keys for the given idKey
     *
     * @param address
     * @returns {Promise<*>}
     */
    getIdentityFromAddress(address: string): Promise<GetIdentityByAddressResponse>;
    /**
     * Get all signing keys for the given idKey
     *
     * @param idKey
     * @returns {Promise<*>}
     */
    getIdentity(idKey: string): Promise<any>;
    /**
     * Get all attestations for the given attestation hash
     *
     * @param attestationHash
     */
    getAttestationsForHash(attestationHash: string): Promise<GetAttestationResponse>;
}
export { BAP_ID };
